@@ -143,6 +143,9 @@ gboolean            rmdir_recursive              (const gchar *directory);
 gboolean            file_is_image                (const gchar *name,
 						  gboolean fast_file_type);
 
+gboolean            image_is_jpeg                (const char *name);
+
+
 gboolean            file_extension_is            (const char *filename, 
 						  const char *ext);
 
@@ -154,6 +157,8 @@ GList *             dir_list_filter_and_sort     (GList *dir_list,
 
 char*               shell_escape                 (const char *filename);
 
+char *              escape_underscore            (const char *name);
+
 char *              get_terminal                 (gboolean with_exec_flag);
 
 char *              application_get_command      (const GnomeVFSMimeApplication *app);
@@ -163,6 +168,8 @@ char *              get_path_relative_to_dir     (const char *filename,
 
 char *              remove_special_dirs_from_path (const char *path);
      
+GnomeVFSURI *       new_uri_from_path             (const char *path);
+
 
 #define errno_to_string() (gnome_vfs_result_to_string (gnome_vfs_result_from_errno ()))
 
