@@ -1533,6 +1533,7 @@ image_list_append (ImageList  *gil,
 	g_return_val_if_fail (pixbuf != NULL, -1);
 
 	image = image_new_from_pixbuf (gil, pixbuf, text, comment);
+
 	return image_list_append_image (gil, image);
 }
 
@@ -1666,6 +1667,7 @@ image_list_clear (ImageList *gil)
 
 	for (l = gil->priv->image_list; l; l = l->next)
 		image_destroy (l->data);
+	g_list_free (gil->priv->image_list);
 
 	gil_free_line_info (gil);
 
@@ -4017,13 +4019,14 @@ image_list_set_image_text (ImageList  *gil,
 		return;
 	
 	image = g_list_nth (gil->priv->image_list, pos)->data;
+
 	gthumb_text_item_configure (image->text,
 				    0, 
 				    0, 
 				    gil->priv->max_image_width, 
 				    DEFAULT_FONT,
 				    text, 
-				    FALSE,
+				    gil->priv->is_editable,
 				    gil->priv->static_text);
 
 	if (gil->priv->frozen) {
