@@ -29,6 +29,8 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
 
+#include "async-pixbuf-ops.h"
+#include "gth-pixbuf-op.h"
 #include "gthumb-window.h"
 #include "pixbuf-utils.h"
 
@@ -71,32 +73,51 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
+static gboolean
+preview_done_cb (GthPixbufOp *pixop,
+		 gboolean     completed,
+		 DialogData  *data)
+{
+	gtk_widget_queue_draw (data->cb_preview_image);
+	g_object_unref (pixop);
+	return FALSE;
+}
+
+
 static void
 apply_changes (DialogData *data, 
 	       GdkPixbuf  *src, 
 	       GdkPixbuf  *dest, 
 	       gboolean    preview)
 {
-	double   cyan_red;
-	double   magenta_green;
-	double   yellow_blue;
-	gboolean preserve_luminosity;
+	double       cyan_red;
+	double       magenta_green;
+	double       yellow_blue;
+	gboolean     preserve_luminosity;
+	GthPixbufOp *pixop;
 
 	cyan_red = gtk_range_get_value (GTK_RANGE (data->cb_cyan_red_hscale));
 	magenta_green = gtk_range_get_value (GTK_RANGE (data->cb_magenta_green_hscale));
 	yellow_blue = gtk_range_get_value (GTK_RANGE (data->cb_yellow_blue_hscale));
 	preserve_luminosity = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->cb_luminosity_checkbutton));
 
-	_gdk_pixbuf_color_balance (src,
-				   dest,
-				   cyan_red,
-				   magenta_green,
-				   yellow_blue,
-				   preserve_luminosity);
+	pixop = _gdk_pixbuf_color_balance (src,
+					   dest,
+					   cyan_red,
+					   magenta_green,
+					   yellow_blue,
+					   preserve_luminosity);
 
-	if (preview)
-		gtk_image_set_from_pixbuf (GTK_IMAGE (data->cb_preview_image), 
-					   dest);
+	if (preview) {
+		g_signal_connect (G_OBJECT (pixop),
+				  "done",
+				  G_CALLBACK (preview_done_cb),
+				  data);
+		gth_pixbuf_op_start (pixop);
+	} else {
+		window_exec_pixbuf_op (data->window, pixop);
+		g_object_unref (pixop);
+	}
 }
 
 
@@ -117,8 +138,10 @@ static void
 cancel_cb (GtkWidget  *widget, 
 	   DialogData *data)
 {
-	if (data->modified)
+	if (data->modified) {
 		image_viewer_set_pixbuf (data->viewer, data->image);
+		window_image_modified (data->window, FALSE);
+	}
 	gtk_widget_destroy (data->dialog);
 }
 
@@ -132,7 +155,6 @@ preview_cb (GtkWidget  *widget,
 
 	preview = gdk_pixbuf_copy (data->image);
 	apply_changes (data, preview, preview, FALSE);
-	image_viewer_set_pixbuf (data->viewer, preview);
 	g_object_unref (preview);
 
 	data->modified = TRUE;
