@@ -25,9 +25,17 @@
 #include <string.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
+#include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-mime-sniff-buffer.h>
 #include <glade/glade.h>
+
+#ifdef HAVE_LIBEXIF
+#include <exif-data.h>
+#include <exif-content.h>
+#include <exif-entry.h>
+#include "jpegutils/jpeg-data.h"
+#endif /* HAVE_LIBEXIF */
+
 #include "file-data.h"
 #include "gconf-utils.h"
 #include "gthumb-window.h"
@@ -59,6 +67,7 @@ typedef struct {
 	GladeXML     *gui;
 
 	GtkWidget    *dialog;
+	GtkWidget    *j_button_box;
 	GtkWidget    *j_apply_to_all_checkbutton;
 
 	int           rot_type;    /* 0, 90 ,180, 270 */
@@ -76,7 +85,7 @@ destroy_cb (GtkWidget  *widget,
 {
 	if (data->file_list != NULL) 
 		g_list_free (data->file_list);
-	g_object_unref (G_OBJECT (data->gui));
+	g_object_unref (data->gui);
 	g_free (data);
 }
 
@@ -95,21 +104,11 @@ add_image_to_button (GtkWidget    *button,
 }
 
 
-static gboolean 
-image_is_jpeg (const char *name)
+static void
+image_loaded_cb (ImageViewer *viewer,
+		 DialogData  *data)
 {
-	const char *result;
-
-	if (eel_gconf_get_boolean (PREF_FAST_FILE_TYPE))
-		result = gnome_vfs_mime_type_from_name_or_default (name, NULL);
-	else 
-		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
-	
-	/* Unknown file type. */
-	if (result == NULL)
-		return FALSE;
-
-	return (strcmp (result, "image/jpeg") == 0);
+	gtk_widget_set_sensitive (data->j_button_box, TRUE);
 }
 
 
@@ -123,6 +122,8 @@ load_current_image (DialogData *data)
 		return;
 	}
 	
+	gtk_widget_set_sensitive (data->j_button_box, FALSE);
+
 	fd = data->current_image->data;
 	image_viewer_load_image (IMAGE_VIEWER (data->viewer), fd->path);
 
@@ -149,6 +150,90 @@ load_next_image (DialogData *data)
 }
 
 
+#ifdef HAVE_LIBEXIF
+
+
+static gboolean
+swap_fields (ExifContent *content,
+	     ExifTag      tag1,
+	     ExifTag      tag2)
+{
+	ExifEntry     *entry1;
+	ExifEntry     *entry2;
+	unsigned char *data;
+	unsigned int   size;
+	
+	entry1 = exif_content_get_entry (content, tag1);
+	if (entry1 == NULL) 
+		return FALSE;
+	
+	entry2 = exif_content_get_entry (content, tag2);
+	if (entry2 == NULL)
+		return FALSE;
+
+	data = entry1->data;
+	size = entry1->size;
+	
+	entry1->data = entry2->data;
+	entry1->size = entry2->size;
+	
+	entry2->data = data;
+	entry2->size = size;
+
+	return TRUE;
+}
+
+
+static void
+swap_xy_exif_fields (const char *filename)
+{
+	JPEGData     *jdata;
+	ExifData     *edata;
+	unsigned int  i;
+
+	jdata = jpeg_data_new_from_file (filename);
+	if (jdata == NULL)
+		return;
+
+	edata = jpeg_data_get_exif_data (jdata);
+	if (edata == NULL) {
+		jpeg_data_unref (jdata);
+		return;
+	}
+
+	for (i = 0; i < EXIF_IFD_COUNT; i++) {
+		ExifContent *content = edata->ifd[i];
+
+		if ((content == NULL) || (content->count == 0)) 
+			continue;
+		
+		swap_fields (content, 
+			     EXIF_TAG_RELATED_IMAGE_WIDTH,
+			     EXIF_TAG_RELATED_IMAGE_LENGTH);
+		swap_fields (content, 
+			     EXIF_TAG_IMAGE_WIDTH,
+			     EXIF_TAG_IMAGE_LENGTH);
+		swap_fields (content, 
+			     EXIF_TAG_PIXEL_X_DIMENSION,
+			     EXIF_TAG_PIXEL_Y_DIMENSION);
+		swap_fields (content, 
+			     EXIF_TAG_X_RESOLUTION,
+			     EXIF_TAG_Y_RESOLUTION);
+		swap_fields (content, 
+			     EXIF_TAG_FOCAL_PLANE_X_RESOLUTION,
+			     EXIF_TAG_FOCAL_PLANE_Y_RESOLUTION);
+	}
+
+	jpeg_data_save_file (jdata, filename);
+
+	exif_data_unref (edata);
+	jpeg_data_unref (jdata);
+}
+
+
+#endif
+
+
 static void
 apply_tran (DialogData *data,
 	    GList      *current_image)
@@ -222,14 +307,11 @@ apply_tran (DialogData *data,
 		e2 = shell_escape (fd->path);
 	
 		line = g_strdup_printf ("jpegtran -copy all %s -outfile %s %s",
-					command, 
-					e1, 
-					e2);
+					command, e1, e2);
+		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
 
 		g_free (e1);
 		g_free (e2);
-
-		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
 		g_free (line);
 
 		if (err != NULL) {
@@ -285,14 +367,11 @@ apply_tran (DialogData *data,
 		e2 = shell_escape (tmp1);
 
 		line = g_strdup_printf ("jpegtran -copy all %s -outfile %s %s",
-					command, 
-					e1, 
-					e2);
+					command, e1, e2);
+		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
 
 		g_free (e1);
 		g_free (e2);
-
-		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
 		g_free (line);
 
 		if (err != NULL) {
@@ -308,20 +387,29 @@ apply_tran (DialogData *data,
 	e2 = shell_escape (fd->path);
 
 	line = g_strdup_printf ("mv -f %s %s", e1, e2);
-
-	g_free (e1);
-	g_free (e2);
-
 	g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
-	g_free (line);
-
-	g_free (tmp1);
-	g_free (tmp2);
 
 	if (err != NULL) {
 		_gtk_error_dialog_from_gerror_run (parent, &err);
-		return;
+
+	} else {
+		GList *list;
+
+#ifdef HAVE_LIBEXIF
+		if ((rot_type == TRAN_ROTATE_90) || (rot_type == TRAN_ROTATE_270))
+			swap_xy_exif_fields (fd->path);
+#endif
+
+		list = g_list_prepend (NULL, fd->path);
+		all_windows_notify_files_changed (list);
+		g_list_free (list);
 	}
+
+	g_free (e1);
+	g_free (e2);
+	g_free (line);
+	g_free (tmp1);
+	g_free (tmp2);
 }
 
 
@@ -420,9 +508,15 @@ rot90_clicked (GtkWidget  *button,
 	}
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+
+	if (src_pixbuf == NULL) 
+		return;
+
 	dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, FALSE);
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
-	g_object_unref (dest_pixbuf);
+
+	if (dest_pixbuf != NULL) 
+		g_object_unref (dest_pixbuf);
 }
 
 
@@ -442,9 +536,15 @@ rot270_clicked (GtkWidget  *button,
 		data->rot_type = get_next_rot (data->rot_type);
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+
+	if (src_pixbuf == NULL) 
+		return;
+
 	dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, TRUE);
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
-	g_object_unref (dest_pixbuf);
+
+	if (dest_pixbuf != NULL) 
+		g_object_unref (dest_pixbuf);
 }
 
 
@@ -452,23 +552,31 @@ static void
 mirror_clicked (GtkWidget  *button, 
 		DialogData *data)
 {
-	ImageViewer  *viewer = IMAGE_VIEWER (data->viewer);
-	GdkPixbuf    *src_pixbuf;
-	GdkPixbuf    *dest_pixbuf;
+	ImageViewer *viewer = IMAGE_VIEWER (data->viewer);
+	GdkPixbuf   *src_pixbuf;
+	GdkPixbuf   *dest_pixbuf;
+
+	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+
+	if (src_pixbuf == NULL) 
+		return;
 
 	if (data->tran_type == TRAN_FLIP) {
 		data->tran_type = TRAN_NONE;
 		data->rot_type = get_next_rot (data->rot_type);
 		data->rot_type = get_next_rot (data->rot_type);
+
 	} else if (data->tran_type == TRAN_MIRROR)
 		data->tran_type = TRAN_NONE;
+
 	else
 		data->tran_type = TRAN_MIRROR;
-
-	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+	
 	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
-	g_object_unref (dest_pixbuf);
+	
+	if (dest_pixbuf != NULL) 
+		g_object_unref (dest_pixbuf);
 }
 
 
@@ -484,15 +592,56 @@ flip_clicked (GtkWidget  *button,
 		data->tran_type = TRAN_NONE;
 		data->rot_type = get_next_rot (data->rot_type);
 		data->rot_type = get_next_rot (data->rot_type);
+
 	} else if (data->tran_type == TRAN_FLIP)
 		data->tran_type = TRAN_NONE;
+
 	else
 		data->tran_type = TRAN_FLIP;
 
 	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+
+	if (src_pixbuf == NULL) 
+		return;
+
 	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, FALSE, TRUE);
 	image_viewer_set_pixbuf (viewer, dest_pixbuf);
-	g_object_unref (dest_pixbuf);
+
+	if (dest_pixbuf != NULL) 
+		g_object_unref (dest_pixbuf);
+}
+
+
+/* called when the "help" button is clicked. */
+static void
+help_cb (GtkWidget  *widget, 
+	 DialogData *data)
+{
+	GError *err;
+
+	err = NULL;  
+	gnome_help_display ("gthumb", "rotate-jpeg", &err);
+	
+	if (err != NULL) {
+		GtkWidget *dialog;
+		
+		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
+						 0,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Could not display help: %s"),
+						 err->message);
+		
+		g_signal_connect (G_OBJECT (dialog), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		
+		gtk_widget_show (dialog);
+		
+		g_error_free (err);
+	}
 }
 
 
@@ -511,7 +660,7 @@ dlg_jpegtran (GThumbWindow *window)
 	GtkWidget   *j_ok_button;
 	GList       *list, *scan;
 
-	list = file_list_get_selection_as_fd (window->file_list);
+	list = gth_file_list_get_selection_as_fd (window->file_list);
 	if (list == NULL) {
 		g_warning ("No file selected.");
 		return;
@@ -532,8 +681,7 @@ dlg_jpegtran (GThumbWindow *window)
 	}
 
 	if (list == NULL) {
-		_gtk_error_dialog_run (GTK_WINDOW (window->app),
-				       _("No JPEG image selected"));
+		g_warning ("No JPEG image selected");
 		return;
 	}
 
@@ -558,6 +706,7 @@ dlg_jpegtran (GThumbWindow *window)
 
 	data->dialog = glade_xml_get_widget (data->gui, "jpeg_rotate_dialog");
 	data->j_apply_to_all_checkbutton = glade_xml_get_widget (data->gui, "j_apply_to_all_checkbutton");
+	data->j_button_box = glade_xml_get_widget (data->gui, "j_button_box");
 
 	j_image_vbox = glade_xml_get_widget (data->gui, "j_image_vbox");
 	j_revert_button = glade_xml_get_widget (data->gui, "j_revert_button");
@@ -565,6 +714,7 @@ dlg_jpegtran (GThumbWindow *window)
 	j_rot_270_button = glade_xml_get_widget (data->gui, "j_rot_270_button");
 	j_v_flip_button = glade_xml_get_widget (data->gui, "j_v_flip_button");
 	j_h_flip_button = glade_xml_get_widget (data->gui, "j_h_flip_button");
+
 	j_help_button = glade_xml_get_widget (data->gui, "j_help_button");
 	j_cancel_button = glade_xml_get_widget (data->gui, "j_cancel_button");
 	j_ok_button = glade_xml_get_widget (data->gui, "j_ok_button");
@@ -605,6 +755,10 @@ dlg_jpegtran (GThumbWindow *window)
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (data->dialog));
+	g_signal_connect (G_OBJECT (j_help_button), 
+			  "clicked",
+			  G_CALLBACK (help_cb),
+			  data);
 	g_signal_connect (G_OBJECT (j_ok_button), 
 			  "clicked",
 			  G_CALLBACK (ok_clicked),
@@ -630,6 +784,11 @@ dlg_jpegtran (GThumbWindow *window)
 			  G_CALLBACK (flip_clicked),
 			  data);
 
+	g_signal_connect (G_OBJECT (data->viewer),
+			  "image_loaded",
+			  G_CALLBACK (image_loaded_cb),
+			  data);
+
 	/* Run dialog. */
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window->app));
